{% import 'system_macros.html' as system %}
<!DOCTYPE html>
<html lang="en" class="h-full" bg-gray-100>
    {% block head %}
    {{system.head(data=page)}}
    <link href="https://cdn.jsdelivr.net/npm/tailwindcss@2.2.19/dist/tailwind.min.css" rel="stylesheet">
    {% endblock %}
<head>

</head>
<body>
    <div class="min-h-full">
        <nav class="bg-gray-800">
          <div class="mx-auto max-w-7xl px-4 sm:px-6 lg:px-8">
            <div class="flex h-16 items-center justify-between">
              <div class="flex items-center">
                <div class="flex-shrink-0">
                  <img class="h-8 w-8" src="https://tailwindui.com/img/logos/mark.svg?color=indigo&shade=500" alt="Your Company">
                </div>
               
                  <div class="ml-10 flex items-baseline space-x-4">
                    <!-- Current: "bg-gray-900 text-white", Default: "text-gray-300 hover:bg-gray-700 hover:text-white" -->
                    {% set active_page = request.path %}
                    {% for page in navigation.primary %}
                      <a href="{{page.url}}" class="{{'rounded-md bg-gray-900 px-3 py-2 text-sm font-medium text-white' if active_page==page.url else 'rounded-md px-3 py-2 text-sm font-medium text-gray-300 hover:bg-gray-700 hover:text-white'}}" aria-current="{{'page' if active_page == page.url else ''}}">{{page.title}}</a>
                    {% endfor %}
                  </div>
                
              </div>
            </div>
          </div>
        </nav>
      
        <header class="bg-white shadow">
          <div class="mx-auto max-w-7xl px-4 py-6 sm:px-6 lg:px-8">
            <h1 class="text-3xl font-bold tracking-tight text-gray-900">{{page.title}}</h1>
          </div>
        </header>
        <main>
          <div class="mx-auto max-w-7xl px-4 py-6 sm:px-6 lg:px-8">
            <!-- Your content -->
            {% block content %}
            {{page.content|safe}}
            {% endblock %} 
          </div>
        </main>
      </div>
</body>
{% block js %}
{% yield 'js' %}
{% endblock %}
</html>
  