{% set input_attrs = input.props %}
{% if input.type not in ('hidden','divider','section','html') %}
<label for="{{input.label_for}}"
       class="{% if input.required %}required {% endif %}form-label">
    {{ input.label }}
</label>
{% endif %}
<textarea id="{{input.label_for}}"
  {% for propname,propval in input_attrs.items() if propval %}
    {{propname}}="{{propval}}"
  {% endfor %}
  cols="50" rows="4"
  id="{{input_attrs.id}}"
  class="form-input {% if input.required %}required {% endif %}{{input_attrs.class}}"
  type="{{ input.type }}"
  placeholder="{{ input_attrs.placeholder or input.label }}"
  name="{{input_attrs.name}}"
  {% if form.type == 'product' %}data-parent="attributes"{% endif %}>
</textarea>

{% if input.required %}
  <small class="input_error_msg {{input.label_for}}">{{input.label or 'This field'}} is required</small>
{% endif %}
