{% set input_attrs = input.props %}
<div class="input-group">
<label for="{{input.label_for}}" 
class="form-label{% if input.required %} required{% endif %}">
{{ input.label|title }}
</label>
<select id="{{input.label_for}}"
type="select"
name="{{input_attrs.name}}"
data-name="{{ input_attrs.name }}"
class="{% if input.required %}required {% endif %}form-input">
{% for option in input.inputs %}
        {% set option_name = option.label %}
        {% set optionvalue = option.props.value or option.label %}
        {% set option_price = optionvalue|babelmoney() if option.price_options else optionvalue %}
        {% set upprice = option_price + form.price.base_price if option.price_increase else None %}
          <option
              {# value="{{option_price if option.price_options else optionvalue}}" #}
              value="{{ option_name }}"
              {% if option.selected %}selected{% endif %}
              {% if input.price_options %} data-variation-price="{{option_price}}"{% endif %}>
              {% if input.price_options %}
              {{ option_name }} - {{option_price}} {% if upprice %} - ${{upprice}}{% endif %}
              {% else %}
              {{ optionvalue}}
              {% endif %}
          </option>
    {% endfor %}
</select>

{% if input.required %}
<small class="input_error_msg {{input.label_for}}">{{input.label or 'This field'}} is required</small>
{% endif %}
</div>
