<div id="media-upload" class="row">
  <div id="dropbox">
    <!-- <ul id="imageList" class="debug row"> </ul> -->
  <div id="upload-btn" class="btn">Upload Images
    <input type="file" name="files" multiple data-parent="files" data-name="{{input.attr_name}}"></div>
  </div>
  <div class="progress-bar"><span class="meter"></span></div>
</div>
