{% set input_attrs = input.props %}
{% if input.show_label and input.type != 'hidden' %}
<label for="{{input.label_for}}"
       class="{% if input.required %}required {% endif %}form-label">
    {{ input.label }}
</label>
{% endif %}
    <input
        tabindex="0"
    value="{{input_attrs.value or ''}}"
    class="form-input {{ input_attrs.class or '' }}{% if input.required %} required{% endif %}"
    type="{{ input.type }}"
    id="{{input_attrs.id or input_attrs.name}}"
    placeholder="{{ input_attrs.placeholder or input.label }}"
    name="{{input_attrs.name}}"
    {% if form.formtype == 'product' %}data-parent="attributes"{% endif %}>
{% if input.required %}
  <small class="input_error_msg {{input.label_for}}">{{input.label or 'This field'}} is required</small>
{% endif %}
